
#include <winsock2.h>
#include "Socket.h"


typedef SOCKET socket_t;

struct Socket::Privates {
	unsigned int refcount;
	socket_t socket;
	Privates()
		: refcount(0)
		, socket(INVALID_SOCKET)
	{
	}
};

void Socket::assign(Privates *p)
{
	if (p) {
		p->refcount++;
	}
	if (privates) {
		if (privates->refcount > 1) {
			privates->refcount--;
		} else {
			delete privates;
		}
	}
	privates = p;
}

Socket::Socket(Socket const &r)
	: privates(0)
{
	assign(r.privates);
}

Socket::~Socket()
{
	assign(0);
}

void Socket::operator = (Socket const &r)
{
	assign(r.privates);
}

bool Socket::isvalid() const
{
	if (privates && privates->socket != INVALID_SOCKET) {
		return true;
	}
	return false;
}

bool Socket::socket(int af, int type, int protocol)
{
	assign(new Privates());
	privates->socket = ::socket(af, type, protocol);
	return isvalid();
}

bool Socket::connect(const struct sockaddr *name, int namelen)
{
	int r = ::connect(privates->socket, name, namelen);
	return r != -1;
}

Socket Socket::accept(struct sockaddr *addr, int *addrlen)
{
	Socket newsock;
	newsock.assign(new Privates());
	newsock.privates->socket = ::accept(privates->socket, addr, addrlen);
	return newsock;
}

int Socket::send(void const *buf, int len, int flags)
{
	return ::send(privates->socket, (char const *)buf, len, flags);
}

int Socket::recv(void *buf, int len, int flags)
{
	return ::recv(privates->socket, (char *)buf, len, flags);
}

void Socket::close()
{
	if (isvalid()) {
		::closesocket(privates->socket);
	}
	assign(0);
}

int Socket::bind(const struct sockaddr *name, int namelen)
{
	return ::bind(privates->socket, name, namelen);
}

int Socket::listen(int backlog)
{
	return ::listen(privates->socket, backlog);
}

int Socket::setsockopt(int level, int optname, const char *optval, int optlen)
{
	return ::setsockopt(privates->socket, level, optname, optval, optlen);
}

bool Socket::sendtcp(void const *ptr, size_t len)
{
	size_t pos = 0;
	while (pos < len) {
		int n = ::send(privates->socket, (char const *)ptr + pos, (int)(len - pos), 0);
		if (n < 1) {
			return false;
		}
		pos += n;
	}
	return true;
}

bool Socket::recvtcp(void *ptr, size_t len)
{
	size_t pos = 0;
	while (pos < len) {
		int n = ::recv(privates->socket, (char *)ptr + pos, (int)(len - pos), 0);
		if (n < 1) {
			return false;
		}
		pos += n;
	}
	return true;
}

